@@ -31,6 +31,6 @@
 
 
 
-/* to suppress autoconf warnings although not used: */
-/* #define HAVE_RX_H 1 */
-/* #define HAVE_GETOPT_H 1 */
+/* do we need the LIBIMPORT ? This toggles whether the local or
+   system wide include files are being used. */
+#undef NEED_IMPORT 
