@@ -22,46 +22,38 @@
 */
 
 /*
-  first include the data from the configuration package if you
-  haven't done so already... we need this to assure the flags
-  like HAVE_RX_H are being set properly.
+  first include the data from the configuration package if not done
+  so already... we need this to assure the flag NEED_LIBIMPORT is
+  being set properly.
 */
-#ifdef PACKAGE
 #include "../config.h"
-#endif /*PACKAGE */
-
 
 
+#if NEED_IMPORT
 /* 
    include file for proper GNUish commandline parsing 
 */
-#if HAVE_GETOPT_H
-/* take system wide include file */
-#include <getopt.h>
-#else
-/* take our local one because system wide one doesn't exist */
 #include "getopt.h"
-#endif /* HAVE_GETOPT_H */
-
-
-
 
 /* 
    include file for the REGEXP functions from the local system if possible,
    otherwise use our local version of it.
 */
-#if HAVE_RX_H
-/* take system wide include file */
-#include <rx.h>
-#else
-/* take our local one because system wide one doesn't exist */
 #include "rx.h"
-#endif /* HAVE_RX_H */
-
-
 
+#else
+/* 
+   include file for proper GNUish commandline parsing 
+*/
+#include <getopt.h>
 
+/* 
+   include file for the REGEXP functions from the local system if possible,
+   otherwise use our local version of it.
+*/
+#include <rx.h>
 
+#endif
 
 
 
