@@ -34,6 +34,7 @@
 
 #include "../config.h"
 #include "sysinc.H"
+#include "../import/import.h"
 #include "logclass.H"
 #include "alert.H"
 #include "extern.H"
@@ -41,93 +42,144 @@
 /*}}}*/
 
 /*{{{  activate / deactivate  */
-void trigger_alert(int nr){
-  if ( entry[nr]->alert_tag != 0 ){
-    entry[nr]->fadestep = fade_base / 3;
-  } else {
-    entry[nr]->fade = 0;
-    entry[nr]->last_fade = 0;
-    entry[nr]->fadestep = fade_base / 3;
-    entry[nr]->alert_tag = gtk_timeout_add (100,
-						 *alert_interrupt,
-						 (gpointer*) nr);
-  }
+/*
+  activates alert for entry number "nr" (parameter). 
+*/
+void 
+trigger_alert(int nr)
+{
+  if ( entry[nr]->alert_tag != 0 )
+    {
+      entry[nr]->fadestep = fade_base / 3;
+    } 
+  else
+    {
+      entry[nr]->fade = 0;
+      entry[nr]->last_fade = 0;
+      entry[nr]->fadestep = fade_base / 3;
+      entry[nr]->alert_tag = gtk_timeout_add (100,
+					      *alert_interrupt,
+					      (gpointer*) nr);
+    }
 }
-void remove_alert(int nr){
-  if ( entry[nr]->alert_tag != 0 ){
-    gtk_timeout_remove(entry[nr]->alert_tag);
-    entry[nr]->alert_tag = 0;
-  }
+/*
+  removes the alert for entry number "nr" (parameter)
+*/
+void 
+remove_alert(int nr)
+{
+  if ( entry[nr]->alert_tag != 0 )
+    {
+      gtk_timeout_remove(entry[nr]->alert_tag);
+      entry[nr]->alert_tag = 0;
+    }
   
   if ( entry[nr]->pixels[0] != 0 && entry[nr]->pixels[1] != 0 )
     gdk_colors_free(cmap, entry[nr]->pixels, 2, 0);
   entry[nr]->pixels[0] = entry[nr]->pixels[1] = 0;
 
   entry[nr]->style = gtk_style_copy(stdstyle);
+  /*
+    if this is the active logfile the "no alert" color is PRELIGHT
+    and not NORMAL !
+  */  
+  if ( nr == active )
+    {
+      entry[nr]->style->bg[GTK_STATE_NORMAL].red = prelight.red;
+      entry[nr]->style->bg[GTK_STATE_NORMAL].green = prelight.green;
+      entry[nr]->style->bg[GTK_STATE_NORMAL].blue = prelight.blue;
+    }  
   gtk_widget_set_style(entry[nr]->button, entry[nr]->style);
   gtk_style_unref(entry[nr]->style);
-
+  
   entry[nr]->fade = entry[nr]->last_fade = entry[nr]->fadestep = 0;
 }
 /*}}}*/
+
 /*{{{  alert_interrupt  */
-int alert_interrupt(gpointer data){ 
+/*
+  the alert interrupt routine that makes the button change it's color
+*/
+int 
+alert_interrupt(gpointer data)
+{ 
   int nr = (int) data;
   entry[nr]->fade += entry[nr]->fadestep;
-  if ( entry[nr]->fade > fade_base ){
-    entry[nr]->fade = fade_base;
-    entry[nr]->fadestep = fade_step;
-  } else if ( entry[nr]->fade < 0 ) remove_alert(nr);
+  if ( entry[nr]->fade > fade_base )
+    {
+      entry[nr]->fade = fade_base;
+      entry[nr]->fadestep = fade_step;
+    } 
+  else if ( entry[nr]->fade < 0 ) 
+    remove_alert(nr);
 
   if ( entry[nr]->fade - entry[nr]->last_fade > ( fade_base / steps ) ||
-       entry[nr]->last_fade - entry[nr]->fade > ( fade_base / steps ) ){
-    entry[nr]->last_fade = entry[nr]->fade;
+       entry[nr]->last_fade - entry[nr]->fade > ( fade_base / steps ) )
+    {
+      entry[nr]->last_fade = entry[nr]->fade;
+      
+      GdkColor normal = {0,0,0,0};
+      GdkColor plight = {0,0,0,0};
+      
+      gfloat alert_fade = (gfloat) ( (gfloat) entry[nr]->fade / fade_base);
+      gfloat orig_fade = (gfloat) ((fade_base - (gfloat) entry[nr]->fade) / fade_base);
+      
+      plight.red = (gushort) ((( alert_fade * alert.red )) + 
+			      ( orig_fade * prelight.red ));
+      plight.green = (gushort) ((( alert_fade * alert.green )) + 
+				( orig_fade * prelight.green ));
+      plight.blue = (gushort) ((( alert_fade * alert.blue )) + 
+			       ( orig_fade * prelight.blue ));
+      
+      normal.red = (gushort) ((( alert_fade * alert.red )) + 
+			      ( orig_fade * base.red ));
+      normal.green = (gushort) ((( alert_fade * alert.green )) + 
+				( orig_fade * base.green ));
+      normal.blue = (gushort) ((( alert_fade * alert.blue )) + 
+			       ( orig_fade * base.blue ));
+      
+      gdk_color_alloc (cmap, &normal);
+      gdk_color_alloc (cmap, &plight);
+      
+      entry[nr]->style = gtk_style_copy(stdstyle);
 
-    GdkColor normal = {0,0,0,0};
-    GdkColor plight = {0,0,0,0};
+      /*
+	if the current entry is the active one the color we are fading back to
+	is the PRELIGHT color !
+      */
+      if ( nr == active )
+	{
+	  entry[nr]->style->bg[GTK_STATE_NORMAL].red = plight.red;
+	  entry[nr]->style->bg[GTK_STATE_NORMAL].green = plight.green;
+	  entry[nr]->style->bg[GTK_STATE_NORMAL].blue = plight.blue;
+	}
+      else
+	{
+	  entry[nr]->style->bg[GTK_STATE_NORMAL].red = normal.red;
+	  entry[nr]->style->bg[GTK_STATE_NORMAL].green = normal.green;
+	  entry[nr]->style->bg[GTK_STATE_NORMAL].blue = normal.blue;
+	}
 
-    gfloat alert_fade = (gfloat) ( (gfloat) entry[nr]->fade / fade_base);
-    gfloat orig_fade = (gfloat) ((fade_base - (gfloat) entry[nr]->fade) / fade_base);
-
-    plight.red = (gushort) ((( alert_fade * alert.red )) + 
-			    ( orig_fade * prelight.red ));
-    plight.green = (gushort) ((( alert_fade * alert.green )) + 
-			    ( orig_fade * prelight.green ));
-    plight.blue = (gushort) ((( alert_fade * alert.blue )) + 
-			    ( orig_fade * prelight.blue ));
-
-    normal.red = (gushort) ((( alert_fade * alert.red )) + 
-			    ( orig_fade * base.red ));
-    normal.green = (gushort) ((( alert_fade * alert.green )) + 
-			    ( orig_fade * base.green ));
-    normal.blue = (gushort) ((( alert_fade * alert.blue )) + 
-			    ( orig_fade * base.blue ));
+      entry[nr]->style->bg[GTK_STATE_PRELIGHT].red = plight.red;
+      entry[nr]->style->bg[GTK_STATE_PRELIGHT].green = plight.green;
+      entry[nr]->style->bg[GTK_STATE_PRELIGHT].blue = plight.blue;
+      entry[nr]->style->bg[GTK_STATE_SELECTED].red = plight.red;
+      entry[nr]->style->bg[GTK_STATE_SELECTED].green = plight.green;
+      entry[nr]->style->bg[GTK_STATE_SELECTED].blue = plight.blue;
+      entry[nr]->style->bg[GTK_STATE_ACTIVE].red = plight.red;
+      entry[nr]->style->bg[GTK_STATE_ACTIVE].green = plight.green;
+      entry[nr]->style->bg[GTK_STATE_ACTIVE].blue = plight.blue;
+      gtk_widget_set_style(entry[nr]->button, entry[nr]->style);
+      gtk_style_unref(entry[nr]->style);
+      
+      if ( entry[nr]->pixels[0] != 0 && entry[nr]->pixels[1] != 0 )
+	gdk_colors_free(cmap, entry[nr]->pixels, 2, 0);
     
-    gdk_color_alloc (cmap, &normal);
-    gdk_color_alloc (cmap, &plight);
-    
-    entry[nr]->style = gtk_style_copy(stdstyle);
-    entry[nr]->style->bg[GTK_STATE_NORMAL].red = normal.red;
-    entry[nr]->style->bg[GTK_STATE_NORMAL].green = normal.green;
-    entry[nr]->style->bg[GTK_STATE_NORMAL].blue = normal.blue;
-    entry[nr]->style->bg[GTK_STATE_PRELIGHT].red = plight.red;
-    entry[nr]->style->bg[GTK_STATE_PRELIGHT].green = plight.green;
-    entry[nr]->style->bg[GTK_STATE_PRELIGHT].blue = plight.blue;
-    entry[nr]->style->bg[GTK_STATE_SELECTED].red = plight.red;
-    entry[nr]->style->bg[GTK_STATE_SELECTED].green = plight.green;
-    entry[nr]->style->bg[GTK_STATE_SELECTED].blue = plight.blue;
-    entry[nr]->style->bg[GTK_STATE_ACTIVE].red = plight.red;
-    entry[nr]->style->bg[GTK_STATE_ACTIVE].green = plight.green;
-    entry[nr]->style->bg[GTK_STATE_ACTIVE].blue = plight.blue;
-    gtk_widget_set_style(entry[nr]->button, entry[nr]->style);
-    gtk_style_unref(entry[nr]->style);
+      entry[nr]->pixels[0] = normal.pixel;
+      entry[nr]->pixels[1] = plight.pixel;
+    }
 
-    if ( entry[nr]->pixels[0] != 0 && entry[nr]->pixels[1] != 0 )
-      gdk_colors_free(cmap, entry[nr]->pixels, 2, 0);
-    
-    entry[nr]->pixels[0] = normal.pixel;
-    entry[nr]->pixels[1] = plight.pixel;
-  }
+  return TRUE;
 }
 /*}}}*/
 
