@@ -62,7 +62,7 @@ popup_dialog(gchar* title, gchar* text, GList* button_list, GList* function_list
 		      label, TRUE, TRUE, 5);
   gtk_widget_show (label);
 
-  GtkWidget* button;
+  GtkWidget* button = NULL;
   gint buttons = g_list_length(button_list);
   for ( gint x = 1 ; x < buttons ; x++ )
     {
