@@ -34,6 +34,7 @@
 
 #include "../config.h"
 #include "sysinc.H"
+#include "../import/import.h"
 #include "logclass.H"
 #include "watchdog.H"
 #include "extern.H"
@@ -42,7 +43,9 @@
 
 /*{{{  Logfile handling (only class0 filters)  */
 /*{{{  activate()  */
-void activate(){
+void 
+activate()
+{
   if ( display_logs == DISABLED ) return;
   /* speedup fadedown for activated logfile */
   entry[active]->fadestep = -1 * ( fade_base / 3 );
@@ -83,12 +86,33 @@ void activate(){
       entry[active]->fd = open(entry[active]->filename, O_RDONLY);
       if ( entry[active]->fd == -1 ){
 	file_error(entry[active]->filename);
+	deactivate();
 	return;
       }
       
-      lseek(entry[active]->fd, -maxtext, SEEK_END);
-      
-      long got = read(entry[active]->fd, read_buffer, maxtext);
+      glong got;
+      fstat( entry[active]->fd,  &status );
+      /*
+	We need to check whether the file is shorter than maxtext
+	bytes because some systems (like FreeBSD) allow a negative
+	index for files...
+      */
+      if ( maxtext < status.st_size )
+	{
+	  /* if the file is bigger than maxtext we should
+	     read maxtext bytes 
+	  */
+	  lseek(entry[active]->fd, -maxtext, SEEK_END);
+	  got = read(entry[active]->fd, read_buffer, maxtext);
+	}
+      else
+	{
+	  /*
+	    otherwise read everything there is
+	  */
+	  lseek(entry[active]->fd, 0, SEEK_SET);
+	  got = read(entry[active]->fd, read_buffer, status.st_size );
+	}
       
       /* Freeze for output: */
       gtk_text_freeze (GTK_TEXT(textwindow));
@@ -103,9 +127,8 @@ void activate(){
 			       GTK_TEXT(textwindow)->vadj->page_size);
       
       // Memorize size of logfile...
-      fstat( entry[active]->fd,  &status );
       entry[active]->last_size = status.st_size;
-      
+
       entry[active]->tag = gtk_timeout_add (entry[active]->interval*100,
 					     *tail_interrupt,
 					     (gpointer*) active);
@@ -118,6 +141,7 @@ void activate(){
       entry[active]->fd = open(entry[active]->filename, O_RDONLY);
       if ( entry[active]->fd == -1 ){
 	file_error(entry[active]->filename);
+	deactivate();
 	return;
       }
       
@@ -146,12 +170,30 @@ void activate(){
     }
 
   }
-  // Set title to button text:
+  /*
+    Set title to button text:
+  */
   gtk_window_set_title (GTK_WINDOW (window), entry[active]->buttontext );  
+  
+  /*
+    If entry is not in alert mode right now, color it's
+    button in PRELIGHT color !
+  */  
+  entry[active]->style = gtk_style_copy(stdstyle);
+  if ( entry[active]->alert_tag == 0 )
+    {
+      entry[active]->style->bg[GTK_STATE_NORMAL].red = prelight.red;
+      entry[active]->style->bg[GTK_STATE_NORMAL].green = prelight.green;
+      entry[active]->style->bg[GTK_STATE_NORMAL].blue = prelight.blue;
+    }  
+  gtk_widget_set_style(entry[active]->button, entry[active]->style);
+  gtk_style_unref(entry[active]->style);
 }
 /*}}}*/
 /*{{{  deactivate()  */
-void deactivate(){
+void 
+deactivate()
+{
   /*
     We need to disable interrupt-routines...
   */
@@ -175,6 +217,18 @@ void deactivate(){
 			   gtk_text_get_length(GTK_TEXT(textwindow)));
   gtk_text_thaw(GTK_TEXT(textwindow));
   gtk_adjustment_set_value(GTK_TEXT(textwindow)->vadj, 0);
+
+
+  /*
+    If entry is not in alert mode right now, color it's
+    button in NORMAL color !
+  */  
+  if ( entry[active]->alert_tag == 0 )
+    {
+      entry[active]->style = gtk_style_copy(stdstyle);
+      gtk_widget_set_style(entry[active]->button, entry[active]->style);
+      gtk_style_unref(entry[active]->style);
+    }
 }
 /*}}}*/
 /*{{{  tail_interrupt  */
@@ -260,7 +314,8 @@ int cat_interrupt(gpointer data){
   gtk_adjustment_set_value(GTK_TEXT(textwindow)->vadj,
 			   GTK_TEXT(textwindow)->vadj->upper - 
 			   old_value);
-  
+
+  return TRUE;
 }
 /*}}}*/
 /*}}}*/
@@ -300,47 +355,93 @@ int reopen_logfile(int i ){
 /*}}}*/
 
 /*{{{  Watchdog (background log watching) on/off  */
-void start_watchdog(){
+/*
+  Startup routine for background log watching
+  neeeds some more niceness and should probably
+  be split up eventually...
+*/
+void 
+start_watchdog()
+{
   int i;
-  for ( i = 0 ; i < syslogs ; i++ ){
-    if ( entry[i]->filterclass & CLASS1_FILTER ){
-      // we need this buffer ONLY for the tail mode !
-      if ( entry[i]->mode == TAIL_FILE ){
-	entry[i]->buffer = new char[maxtext];
-      }
-      entry[i]->rbuffer = new char[entry[i]->chunk];
-      entry[i]->length = 0;
+  for ( i = 0 ; i < syslogs ; i++ )
+    {
       
-      entry[i]->fd = open(entry[i]->filename, O_RDONLY);
-      
-      if ( entry[i]->mode == TAIL_FILE ){
-	lseek(entry[i]->fd, -maxtext, SEEK_END);
-	entry[i]->length = read(entry[i]->fd, entry[i]->buffer, maxtext);
-      } else {
-	entry[i]->length = read(entry[i]->fd, entry[i]->rbuffer, maxtext);
-      }  
-      
-      if ( entry[i]->mode == TAIL_FILE ){
-	// Memorize size of logfile...
-	fstat( entry[i]->fd,  &status );
-	entry[i]->last_size = status.st_size;
-      } else {
-	close(entry[i]->fd);
-      }
-      
-      entry[i]->wd_tag = gtk_timeout_add (entry[i]->interval*100,
-					   *watchdog_interrupt,
-					   (gpointer*) i);
-      
-    } else {
-      if ( entry[i]->buffer != NULL ) delete entry[i]->buffer;
-      entry[i]->buffer = NULL;
-      if ( entry[i]->rbuffer != NULL ) delete entry[i]->rbuffer;
-      entry[i]->rbuffer = NULL;
+      if ( entry[i]->filterclass & CLASS1_FILTER ){
+	// we need this buffer ONLY for the tail mode !
+	if ( entry[i]->mode == TAIL_FILE ){
+	  entry[i]->buffer = new char[maxtext];
+	}
+	entry[i]->rbuffer = new char[entry[i]->chunk];
+	entry[i]->length = 0;
+	
+	entry[i]->fd = open(entry[i]->filename, O_RDONLY);
+	
+	if ( entry[i]->mode == TAIL_FILE )
+	  {
+	    
+	    fstat( entry[i]->fd,  &status );
+	    
+	    /*
+	      We need to check whether the file is shorter than maxtext
+	      bytes because some systems (like FreeBSD) allow a negative
+	      index for files...
+	    */
+	    if ( maxtext < status.st_size )
+	      {
+		/* if the file is bigger than maxtext we should
+		   read maxtext bytes 
+		*/
+		lseek(entry[i]->fd, -maxtext, SEEK_END);
+		entry[i]->length = read(entry[i]->fd, entry[i]->buffer, maxtext);
+	      }
+	    else
+	      {
+		/*
+		  otherwise read everything there is
+		*/
+		lseek(entry[i]->fd, 0, SEEK_SET);
+		entry[i]->length = read(entry[i]->fd, entry[i]->buffer, status.st_size );
+	      }
+	    
+	    // Memorize size of logfile...
+	    entry[i]->last_size = status.st_size;
+	    
+	    
+	  }
+	else
+	  {
+	    
+	    entry[i]->length = read(entry[i]->fd, entry[i]->rbuffer, maxtext);  
+	    close(entry[i]->fd);
+	    
+	  }
+	
+	
+	entry[i]->wd_tag = gtk_timeout_add (entry[i]->interval*100,
+					    *watchdog_interrupt,
+					    (gpointer*) i);
+	
+      } 
+      else
+	{
+	  
+	  
+	  if ( entry[i]->buffer != NULL ) delete entry[i]->buffer;
+	  entry[i]->buffer = NULL;
+	  if ( entry[i]->rbuffer != NULL ) delete entry[i]->rbuffer;
+	  entry[i]->rbuffer = NULL;
+	  
+	  
+	  
+	  
+	}
+
     }
-  }
 }
-void stop_watchdog(){
+void 
+stop_watchdog()
+{
   int i;
   for ( i = 0 ; i < syslogs ; i++ )
     if ( entry[i]->filterclass & CLASS1_FILTER ){
@@ -360,7 +461,14 @@ void stop_watchdog(){
 /*}}}*/
 
 /*{{{  watchdog_interrupt  */
-int watchdog_interrupt(gpointer data){ 
+/*
+  as the name suggests this is the interrupt routine that does the background
+  log watching... it is responsible for keeping the buffers up-to date and
+  it even displays the text if data (the number of the entry) matches active.
+*/
+int 
+watchdog_interrupt(gpointer data)
+{ 
   int i = (int) data;
 
   if ( entry[i]->mode == TAIL_FILE ){
@@ -418,97 +526,128 @@ int watchdog_interrupt(gpointer data){
       
     }
     
-    long a;
-    int f;
-
     /*
       Now search for Class 1 Filters and take according actions
     */
+/*{{{  Class1 Filter stuff  */
+    glong start = 0;
+    glong stop = 0;
     
-    /* NOTICE */
-    for ( a = 0 ; a < got ; a++ ){
-      f = 0;
-      while ( entry[i]->filter[f] != NULL ){
-	if ( entry[i]->filter[f]->mode & NOTICE ){
-	  if ( does_match(entry[i]->rbuffer, entry[i]->filter[f]->pattern,
-			  a, got ) == TRUE ){
-	    long b,c;
-	    b = c = a;
-	    while ( b > 0 && entry[i]->rbuffer[b] != '\n' && entry[i]->rbuffer[b] != '\r' ) b--;
-	    if ( b != 0 ) b++;
-	    while ( c < got && entry[i]->rbuffer[c] != '\n' && entry[i]->rbuffer[c] != '\r' ) c++;
+    while ( start < got &&
+	    entry[i]->filter != NULL )
+      {
+	
+	stop = match_all( entry[i]->filter, 
+			  CLASS1_FILTER, 
+			  entry[i]->rbuffer, 
+			  start, 
+			  got);
+	
+	gint action = 0;
+	gint x = 0;
+	while ( entry[i]->filter[x] != NULL )
+	  {
 	    
-	    gchar* string = new gchar[(c-b)+4];
-	    long d = 0;
-	    for ( ; b < c ; b++ ) string[d++] = entry[i]->rbuffer[b];
-	    string[d++] = 0;
+	    if ( entry[i]->filter[x]->regex_status >= 0 )
+	      {
+		action |= entry[i]->filter[x]->mode;
+	      }
 	    
-	    notice_alert(entry[i]->buttontext , string);
-	    delete string;
-	    play(NOTICE);
+	    x++;
 	  }
-	}
-	f++;
-      }
-    }
-    
-    /* ALERT */
-    for ( a = 0 ; a < got ; a++ ){
-      f = 0;
-      while ( entry[i]->filter[f] != NULL ){
-	if ( entry[i]->filter[f]->mode & ALERT ){
-	  if ( does_match(entry[i]->rbuffer, entry[i]->filter[f]->pattern,
-			  a, got ) == TRUE ){
-	    trigger_alert(i);
-	    play(ALERT);
-	  }
-	}
-	f++;
-      }
-    }
-   
-    /* UNICONIFY */
-    for ( a = 0 ; a < got ; a++ ){
-      f = 0;
-      while ( entry[i]->filter[f] != NULL ){
-	if ( entry[i]->filter[f]->mode & UNICONIFY ){
-	  if ( does_match(entry[i]->rbuffer, entry[i]->filter[f]->pattern,
-			  a, got ) == TRUE ){
-	    /* This maps the window, which also de-iconifies it according to ICCCM. */
-	    gdk_window_show (GTK_WIDGET (window)->window);
-	    gdk_window_raise (GTK_WIDGET (window)->window);
-	    play(UNICONIFY);
-	  }
-	}
-	f++;
-      }
-    }
+	
+	/*
+	  mask out things that are not CLASS1_FILTERS,
+	  afterwards actions has the bit for every applicable
+	  CLASS1_FILTER set.
+	*/
+	action &= CLASS1_MASK;
+	
+	/*
+	  if action is not 0 we need to do something about
+	  that line, so here is our routine for that:
+	*/
+	
+	if ( action != 0 )
+	  {
+	    gchar* line = NULL;
 
-    /* EXECUTE */
-    for ( a = 0 ; a < got ; a++ ){
-      f = 0;
-      while ( entry[i]->filter[f] != NULL ){
-	if ( entry[i]->filter[f]->mode & EXECUTE ){
-	  if ( does_match(entry[i]->rbuffer, entry[i]->filter[f]->pattern,
-			  a, got ) == TRUE ){
-	    long b,c;
-	    b = c = a;
-	    while ( b > 0 && entry[i]->rbuffer[b] != '\n' && entry[i]->rbuffer[b] != '\r' ) b--;
-	    if ( b != 0 ) b++;
-	    while ( c < got && entry[i]->rbuffer[c] != '\n' && entry[i]->rbuffer[c] != '\r' ) c++;
-	    
-	    gchar* string = new gchar[(c-b)+4];
-	    long d = 0;
-	    for ( ; b < c ; b++ ) string[d++] = entry[i]->rbuffer[b];
-	    string[d++] = 0;
-	    
-	    execute_program(i, f , string);
-	    delete string;
+	    /*
+	      These filters need the line that triggered the match
+	    */
+	    if ( action & NOTICE ||
+		 action & EXECUTE )
+	      {
+		line = new gchar[(stop-start)+5];
+		
+		gint a = 0;
+		gint b = start;
+		while ( b < stop )
+		  {
+
+		    if ( entry[i]->rbuffer[b] != '\r' &&
+			 entry[i]->rbuffer[b] != '\n' )
+		      {
+			line[a++] = entry[i]->rbuffer[b++];
+		      }
+		    else
+		      {
+			b++;
+		      }
+		    
+		  }
+		line[a] = 0;
+		
+	      }
+
+	    if ( action & NOTICE )
+	      {
+		notice_alert(entry[i]->buttontext , line);
+		play(NOTICE);
+	      }
+
+	    if ( action & ALERT )
+	      {
+		trigger_alert(i);
+		play(ALERT);
+	      }
+
+	    if ( action & UNICONIFY )
+	      {
+		/* This maps the window, which also de-iconifies it according to ICCCM. */
+		gdk_window_show (GTK_WIDGET (window)->window);
+		gdk_window_raise (GTK_WIDGET (window)->window);
+		play(UNICONIFY);
+	      }
+
+	    if ( action & EXECUTE )
+	      {
+		x = 0;
+		while (  entry[i]->filter[x] != NULL )
+		  {
+		    
+		    if ( entry[i]->filter[x]->regex_status >= 0 
+			 && entry[i]->filter[x]->mode & EXECUTE )
+		      {
+			execute_program(i, x , line);
+		      }
+		    
+		    x++;
+		  }
+	      }
+
+	    /* 
+	       now the line can be freed again
+	    */
+	    if ( line != NULL ) delete line;
 	  }
-	}
-	f++;
+
+	start = stop + 1;
       }
-    }
+/*}}}*/
+    /*
+      Alright - the CLASS1 Filter stuff is ready now...
+    */
 
 
     /*
@@ -588,10 +727,23 @@ void replace_text(char* buffer, long amount){
 }
 /*}}}*/
 /*{{{  error subroutines for watchdog  */
-void watchdog_file_error(gint i){
+/*
+  This routine handles file access errors during watchdog startup
+  and interrupts.
+
+  The parameter is the number of the entry that caused the problems
+ 
+  - If the number equals te active logfile, deactivate is called
+  - the watchdog interrupt for this entry is being removed
+  - the buffers are freed
+  and a file error dialog box is being popped up
+*/
+void 
+watchdog_file_error(gint i)
+{
   entry[i]->got = -1;      
   if ( i == active ) deactivate();
-  if ( entry[i]->wd_tag ) gtk_timeout_remove(entry[i]->wd_tag);
+  if ( entry[i]->wd_tag != 0 ) gtk_timeout_remove(entry[i]->wd_tag);
   entry[i]->wd_tag = 0;
   if ( entry[i]->buffer != NULL ) delete entry[i]->buffer;
   entry[i]->buffer = NULL;
